/*
LCD monitor and Web uploader for WM series wind sensor

 This code includes modified sample code of SoftwareSerial, LiquidCrystal Library and Repeating Web client.

 Supported model: WM7-RPEA
 Set communication mode on WM sensor to S3A or S32

created 2 Feb 2017
 by Steve Nishiarai
*/

#define VERSION "171208"

// include the library code:
#include <LiquidCrystal.h>
#include <SoftwareSerial.h>
#include <stdlib.h>

/*--------------------------------------------------------------------------------
  Data receiving, sending and display buffers
*/
//set boot up ID (11 letters are used for normal mode, 15 letters are used for test mode)
// When the letter next to C is more or equal to 8 then 2o3 check system is cancelled.
char DTMFhex[] = "C8" VERSION "0000000" ;

char Status = '0' ;
char Temp[] = "+00.00" ;
char Id ; 
char TimeStamp[] = "0000" ;
char PrvTimeStamp[] = "0000" ;

// last time you attempted connect to the server, in milliseconds
unsigned long lastAttemptTime = 0;
// state of the connection last time through the main loop
boolean lastConnected = false;
// delay between updates, in milliseconds
const unsigned long postingInterval = 10000;
// state of the connection success return from the httpRequest
boolean ConnectionSuccess = false;

//default value for average and maximum wind speed and wind direction number ID:C HttpRequest
char AvrWin[] = "     " ;
char MaxWin[] = "     " ;
char AvrDir[] = "   " ;
char MaxDir[] = "   " ;
//wind direction buffer for HTTP upload
char AvrDirNum = '1' ;
char MaxDirNum = '1' ;
//Instantaneous wind direction and speed buffer
char InstDir[] = "   " ;
char InstWin[] = "00.00" ;

//LCD display buffer for test mode
char DispBuff[] = "                " ;
char DspPointer = 0 ;
//last time when the Average and Maximum wind information was uploaded
unsigned long lastAvrWindTime = 0 ;

//flag for blinking viatl sign
char BlinkFlag = 0 ;
//------------------------------------------------------------------------------

/*--------------------------------------------------------------------------------
  Mode selections

  Use A2,A3,A4 pin for selecting mode. These pins are read as lower digit of ModeSel and each bits are reversed.
  Keep these pins open when not in the test mode.

  When ModeSel4 then web connection test will performed and show the received result on LCD.

|A4|A3|A2|
|--|--|Gd| ModeSel1 (no web connection and show angle in degree)
|--|Gd|--| ModeSel2 (show angle in degree)
|--|Gd|Gd| ModeSel3 (not used)
|Gd|--|--| ModeSel4 (web connection test)
|Gd|--|Gd| ModeSel5 (no web connection = Monitor mode)
-- :open, Gd :Connect to GND
*/
//test mode register (0 = not test mode)
char ModeSel ;
//initialize TestStatus for making Web test only once
unsigned char TestStatus = 0 ;
//----------------------------------------------------------------------------

/*------------------------------------------------------------------------------
  Watchdog timer.

  If the loop function does not run in 8 seconds then watchdog timer will reset the system.
  Watch dog timer function is disabled when the A5 pin is connected to the GND.
*/
#include <avr/wdt.h>
// Print loop time for debugging the watch dog timing
//#define DEBUG_WDT
#ifdef DEBUG_WDT
unsigned long Looptime = 0 ;
#endif
//------------------------------------------------------------------------------

/*------------------------------------------------------------------------------
  Software serial multiple serial test
 
 Receives from the hardware serial, sends to software serial.
 Receives from software serial, sends to hardware serial.
 
 The circuit: 
 * RX is digital pin 2 (connect to TX of other device)
 * TX is digital pin 3 (connect to RX of other device)
 
 created back in the mists of time
 modified 9 Apr 2012
 by Tom Igoe
 based on Mikal Hart's example
 
 This example code is in the public domain.

 Use software serial for WM interface and use hardware serial for debugging purposes.
 by Stieve Nishiarai
*/
SoftwareSerial mySerial(2, 3); // RX, TX
//------------------------------------------------------------------------------

/*
  LiquidCrystal Library - Hello World
 
 Demonstrates the use a 16x2 LCD display.  The LiquidCrystal
 library works with all LCD displays that are compatible with the 
 Hitachi HD44780 driver. There are many of them out there, and you
 can usually tell them by the 16-pin interface.
 
 This sketch prints "Hello World!" to the LCD
 and shows the time.
 
  The circuit:
 * LCD RS pin to digital pin 9
 * LCD Enable pin to digital pin 8
 * LCD D4 pin to digital pin 7
 * LCD D5 pin to digital pin 6
 * LCD D6 pin to digital pin 5
 * LCD D7 pin to digital pin 4
 * LCD R/W pin to ground
 * 10K resistor:
 * ends to +5V and ground
 * wiper to LCD VO pin (LCD pin 3)

 Library originally added 18 Apr 2008
 by David A. Mellis
 library modified 5 Jul 2009
 by Limor Fried (http://www.ladyada.net)
 example added 9 Jul 2009
 by Tom Igoe
 modified 22 Nov 2010
 by Tom Igoe
 
 This example code is in the public domain.

 http://www.arduino.cc/en/Tutorial/LiquidCrystal
 --------------
 modified 18 Mar 2015
 by Steve Nishiarai
*/

/*----------------------------------------------------------------
  Repeating Web client
 
 This sketch connects to a web server and makes a request
 using a Wiznet Ethernet shield. You can use the Arduino Ethernet shield, or
 the Adafruit Ethernet shield, either one will work, as long as it's got
 a Wiznet Ethernet module on board.
 
 This example uses DHCP, to get IP address, and DNS address.
 This example uses DNS, by assigning the Ethernet client with a MAC address, IP address, and DNS address.
 
 Circuit:
 * Ethernet shield attached to pins 10, 11, 12, 13
 
 created 19 Apr 2012
 by Tom Igoe
 
 http://arduino.cc/en/Tutorial/WebClientRepeating
 This code is in the public domain.
 
 --------------
 modified 28 Oct 2012
 by Steve Nishiarai
*/
// include the library code:
#include <SPI.h>
#include <Ethernet.h>

// fill in your server domain name here:
//char server[] = "yourdomain.jp";
char server[] = "parawind.net";

//fill in your directory where your target CGI located under your domain:
//char dir[] = "location/" ;
char dir[] = "local/test/" ;

//Assign interval time to send status after Average and Maximum Wind data sent in milli second
#define INTERVAL 60000
//60 seconds

// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(9, 8, 7, 6, 5, 4);

// assign a MAC address for the ethernet controller.
// fill in your address here:
//byte mac[] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
byte mac[] = { 0x84, 0x83, 0xFE, 0xFF, 0x00, 0x02 }; //MAC address adjusted for #2

/*
//------------------->>IP address information when not using DHCP
IPAddress ip(0,0,0,0);

// fill in your Domain Name Server address here:
IPAddress myDns(0,0,0,0);
//------------------<<IP address information
*/

// initialize the library instance:
EthernetClient client;
// Print HTTP data for debugging
//#define DEBUG_HTTP
//----------------------------------------------------------------

void setup()
{
  // reset the WDT timer
  wdt_reset();
  // Disable watchdog timer
  WDTCSR |= (1<<WDCE) | (0<<WDE);

  //Initialize mode pins
  pinMode (A1, INPUT);
  digitalWrite(A1, HIGH);  // set pullup on pin A1
  pinMode (A2, INPUT);
  digitalWrite(A2, HIGH);  // set pullup on pin A2
  pinMode (A3, INPUT);
  digitalWrite(A3, HIGH);  // set pullup on pin A3
  pinMode (A4, INPUT);
  digitalWrite(A4, HIGH);  // set pullup on pin A4
  pinMode (A5, INPUT);
  digitalWrite(A5, HIGH);  // set pullup on pin A5

  //read test mode pins
  ModeSel = 0 ;
  if ( digitalRead(A2) == 0 ) { ModeSel |= 0x01 ;}
  if ( digitalRead(A3) == 0 ) { ModeSel |= 0x02 ;}
  if ( digitalRead(A4) == 0 ) { ModeSel |= 0x04 ;}

  // start serial port:
  Serial.begin(9600);
  // set the data rate for the SoftwareSerial port
  mySerial.begin(9600);

  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);
  // Print a message to the LCD.
  lcd.print(F("RPEA"));
  lcd.print(VERSION);
  lcd.setCursor(0, 1);
  lcd.print(F("MAC"));
  for (char i=0; i<6; i++)
  {
    char c[6] ;
    sprintf ( c, "%02X", mac[i] ) ;
    lcd.print(c);
  }
  //put carsor to mode display place
  lcd.setCursor(10, 0);

  //show mode selection
  lcd.print(F(" Mode"));
  char c = bin2ascii( ModeSel ) ;
  lcd.print(c);

  //if mode 0,2,4
  if ( ModeSel == 0 || ModeSel == 2 || ModeSel == 4 )
  {
    // give the ethernet module time to boot up:
    delay(1000);
    //put carsor to mode display place
    lcd.setCursor(10, 0);
//------------------------------------------------------------------>>get IP address and start Ethernet when using DHCP
    lcd.print(F(" DHCP "));
    if (Ethernet.begin(mac) == 0)
    {
      Serial.println(F("Failed to configure Ethernet using DHCP"));
      // no point in carrying on, so do nothing forevermore:
      while(true);
    }
//------------------------------------------------------------------<<get IP address and start Ethernet when using DHCP

//------------------------------------------------------------------>>Start Ethernet with fixed IP address when not using DHCP
//    lcd.print(" FIXIP");
//    // give the ethernet module time to boot up:
//    delay(1000);
//    Ethernet.begin(mac, ip, myDns) ; 
//------------------------------------------------------------------<<Start Ethernet with fixed IP address when not using DHCP

    //show directory under the domain
    lcd.setCursor(0, 1);
    lcd.print("                ");
    lcd.setCursor(0, 1);
    lcd.print(dir);

    // give the Ethernet shield few seconds to initialize:
    delay(2000);
  //--------------------------------------------------------------
  
    // print the Ethernet board/shield's IP address:
    Serial.print("My IP address: ");
    Serial.println(Ethernet.localIP());
    lcd.setCursor(0, 0);
    lcd.print("My IP address   ");
    lcd.setCursor(0, 1);
    lcd.print("                ");
    lcd.setCursor(0, 1);
    lcd.print(Ethernet.localIP());
    delay(1000);
  
    //Upload the startup indication repetitively until its success
    int a = 1 ;
    while (!ConnectionSuccess)
    {
      lcd.setCursor(0, 0);
      lcd.print("Connect attempt");
      lcd.print(a);
      httpRequest();
      //wait for posting interval
      while (millis() - lastAttemptTime <= postingInterval) {}
      a++;
    }
    //clear the first line
    lcd.setCursor(0, 0);
    lcd.print("                ");
    if ( digitalRead(A5) == 1 )
    {
      watchdogSetup();
    }
  }
	//Clear RX buffer since the WM sentence may disrupted while in the setup process.
	ClearRXbuf();
}

void watchdogSetup(void)
{
  cli(); // disable all interrupts
  // reset the WDT timer
  wdt_reset();
  // Enter Watchdog Configuration mode:
  WDTCSR |= (1<<WDCE) | (1<<WDE);
  // Set Watchdog settings: 8 seconds
  WDTCSR = (1<<WDIE) | (1<<WDE) | (1<<WDP3) | (0<<WDP2) | (0<<WDP1) | (1<<WDP0);
  sei();
}

void software_reset()
{
  wdt_disable();
  wdt_enable(WDTO_15MS);
  while (1) {}
}

void loop()
{
#ifdef DEBUG_WDT
  Looptime = millis() - Looptime ;
  if ( Looptime > 1000 )
  {
    Serial.print(F("Debug "));
    Serial.print(millis()) ;
    Serial.print(F(" "));
    Serial.print(Looptime) ;
    Serial.println();
  }
  Looptime = millis() ;
#endif
  if ( digitalRead(A5) == 1 )
  {
    //Make sure the proccess is in loop
    wdt_reset();
  }
  //--- if WM connection test mode then 
  if ( ModeSel == 4 )
  {
    if ( TestStatus == 0 )
    {
      Serial.println(F("Test mode"));
      lcd.print(F("Test mode"));
      lcd.setCursor(0, 1);
      // close any connection before send a new request.
      client.stop();
      // if there's a successful connection:
      if (client.connect(server, 80))
      {
#ifdef DEBUG_HTTP
        Serial.println(server);
        Serial.println(F("connecting....."));
#endif
        // send the HTTP directory:
        client.print(F("GET /"));
        client.print(dir);
        client.print(F("wmtest.html"));
        client.println(F(" HTTP/1.1"));
        client.print(F("Host: "));
        client.println(server);
        client.println(F("User-Agent: arduino-ethernet"));
        client.println(F("Connection: close"));
        client.println();
#ifdef DEBUG_HTTP
        Serial.println(F("HTTP test request sent."));
#endif
        ConnectionSuccess = true ;
        TestStatus = 1 ;
      } 
      else
      {
        // if you couldn't make a connection:
#ifdef DEBUG_HTTP
        Serial.println(F("connection failed"));
#endif
        ConnectionSuccess = false;
      }
      lcd.setCursor(0, 1);
    }
    if (client.available())
    {
      char c = client.read();
#ifdef DEBUG_HTTP
      Serial.write(c);
#endif
      //put letter to the LCD display buffer
      if ( c == '\n' )
      {
        lcd.setCursor(0, 0);
        for ( char i = 0; i < 16; i++ )
        {
          lcd.print(DispBuff[i]) ;
        }
        //clear the 2nd line
        lcd.setCursor(0, 1);
        lcd.print(F("                "));
        lcd.setCursor(0, 1);
        DspPointer = 0 ;
      }
      else if ( DspPointer < 16 )
      {
        DispBuff[DspPointer] = c ;
        lcd.print(c) ;
        DspPointer++ ;
      }
    }
  }
  //--- end of WM connection test mode
  else
  {
    // if there's incoming data from the net connection.
    // send it out the serial port.
    if (client.available())
    {
      char c = client.read();
      //Serial.print(c);
    }
  
    // if there's no net connection, but there was one last time
    // through the loop, then stop the client:
    if (!client.connected() && lastConnected)
    {
      Serial.println();
      Serial.println("disconnecting.");
      client.stop();
    }
    //Read Wind Sensor data
    ReadWMsensor() ;
    //Convert wind angle to cardinal wind direction (when not test mode 2)
    char DirNum = ConvertDir( InstDir ) ;
    AvrDirNum = ConvertDir( AvrDir ) ;
    MaxDirNum = ConvertDir( MaxDir ) ;

    //Show current wind
    // set the cursor to column 0, line 0
    lcd.setCursor(0, 0);
    //Display received data
    lcd.print(InstDir);
    lcd.print(InstWin);
    lcd.print(" ");
    //clear the last number of temperature
    Temp[5] = ' ' ;
    lcd.print(Temp);
    //Show status
    lcd.print(Status);

    // set the cursor to column 0, line 1
    lcd.setCursor(0, 1);
    //Display received data
    lcd.print(AvrDir);
    lcd.print(AvrWin);
    lcd.print(MaxDir);
    lcd.print(MaxWin);

    //if mode 0,2 then access to the web. (Mode4 never comes here)
    if ( ModeSel == 0 || ModeSel == 2 )
    {
       //if the time stamp has been changed
      if ( TimeStamp[0] != PrvTimeStamp[0] || TimeStamp[1] != PrvTimeStamp[1] || TimeStamp[2] != PrvTimeStamp[2] || TimeStamp[3] != PrvTimeStamp[3] )
      {
        //Send average and maximum wind direction and speed to the Web
        // send HTTP request
        // if 1 set of data comes from Serial, then send it over net
        // if you're not connected, and ten seconds have passed since
        // your last attempt of connection, then connect again and send data:
        if((millis() - lastAttemptTime > postingInterval))
        {
          //Round off the second digit after the decimal point of the average wind speed value
          char WindSpeed[4] ;
          sprintf( WindSpeed, "%c%c%c%c", AvrWin[0], AvrWin[1], AvrWin[3], AvrWin[4] ) ;
          int i = atoi( WindSpeed ) ;
          i += 5 ;
          sprintf( AvrWin, "%05d", i ) ;
          //Round off the maximum wind value
          sprintf( WindSpeed, "%c%c%c%c", MaxWin[0], MaxWin[1], MaxWin[3], MaxWin[4] ) ;
          i = atoi( WindSpeed ) ;
          i += 5 ;
          sprintf( MaxWin, "%05d", i ) ;

          //put wind data to upload buffer that resemble with DTMF buffer
          DTMFhex[0] = Id ;
          DTMFhex[1] = AvrDirNum;
          DTMFhex[2] = AvrWin[1];
          DTMFhex[3] = AvrWin[2];
          DTMFhex[4] = AvrWin[3];
          DTMFhex[5] = MaxDirNum;
          DTMFhex[6] = MaxWin[1];
          DTMFhex[7] = MaxWin[2];
          DTMFhex[8] = MaxWin[3];

          Serial.println(F("Wind data uploaded"));
          httpRequest();
          //renew the time of last update of the wind information
          lastAvrWindTime = millis();
          //renew timestamp
          PrvTimeStamp[0] = TimeStamp[0] ;
          PrvTimeStamp[1] = TimeStamp[1] ;
          PrvTimeStamp[2] = TimeStamp[2] ;
          PrvTimeStamp[3] = TimeStamp[3] ;
        }
      }
      //Telemetry upload process from here
      //telemetry is upload 1 minute after the wind data uploaded
      //if the current time is after the time assigned by INTERVAL from last Average and Maximum Wind data send then send status
      if ( lastAvrWindTime != 0 && millis() - lastAvrWindTime > INTERVAL )
      {
        //reset Wind data updated time
        lastAvrWindTime = 0 ;
        //telemetry ID is always F
        DTMFhex[0] = 'F' ;
        //declare hexa code buffer for temperature data
        char HTmp[] = "  " ;
        //reverse calculation of temperature to AD number
        float ADnum = atof( Temp ) ;
        ADnum = ( 0.0195 * ADnum + 0.4 ) * 256 / 1.25 ;
        //Convert to integer
        int i = ADnum ;
        //Convert temperature to hexa code
        sprintf ( HTmp, "%02X", i ) ;

        DTMFhex[1] = '0' ;
        DTMFhex[2] = '0' ;
        DTMFhex[3] = '0' ;
        DTMFhex[4] = '0' ;
        DTMFhex[5] = '0' ;
        //set status code to the first letter of MaxWin
        if ( Status == 'A' ) { DTMFhex[6] = '0' ;}
        if ( Status == 'E' ) { DTMFhex[6] = '1' ;}
        if ( Status == 'S' ) { DTMFhex[6] = '2' ;}
        if ( Status == 'U' ) { DTMFhex[6] = '3' ;}
        if ( Status == 'V' ) { DTMFhex[6] = '4' ;}
        //put temperature data
        DTMFhex[7] = HTmp[0] ;
        DTMFhex[8] = HTmp[1] ;
        // if you're not connected, and ten seconds have passed since
        // your last attempt of connection, then connect again and send data:
        if((millis() - lastAttemptTime > postingInterval))
        {
          httpRequest();
        }
      }
      // store the state of the connection for next time through
      lastConnected = client.connected();
    }
    //show vital sign with *
    lcd.setCursor(14, 0);
    if (( BlinkFlag & 0x1 ) == 0 )
    {
      lcd.print(F("*"));
    }
    BlinkFlag ^= 0x1 ;
  }
}

/*
Read Wind Sensor data

;------------------------------------------------------------------------------
;WM data format
;        1 2 3   4 5 6    7    8 9   1011  12   13
;        | | |   | | |    |    | |   | |   |    |
; %WIMWV,x,R,x.x,c,c,cx.x,hhhh,x,x.x,x,x.x,hhhh*hh<CR><LF>
;------------------------------------------------------------------------------
;
;1. Wind Angle, 0 to 359 degrees
;2. Reference, R = Relative, T = True
;3. Wind Speed
;4. Wind Speed Units, K/M/N (fixed to M)
;5. Status, A = Data Valid, V = Data Invalid, E = Measurement data overflow occurred, S = Sync loss occurred, U = US pulse not detected
;6. Temperature with sign(+-)
;7. Time Stamp
;8. Average wind Angle, 0 to 359 degrees
;9. Average wind speed
;10. Maximum instantaneous wind Angle, 0 to 359 degrees
;11. Maximum instantaneous wind speed
;12. Time of acquisition
;13. Checksum
*/
void ReadWMsensor()
{
  //Change indication to wind data
  Id = '0' ;
  char prvc = ' ' ;
  //initialize the sequence
  int Sequence = -1 ;
  //clear the right two letters of wind direction to space
  InstDir[2] = ' ' ; InstDir[1] = ' ' ;
  AvrDir[2] = ' ' ; AvrDir[1] = ' ' ;
  MaxDir[2] = ' ' ; MaxDir[1] = ' ' ;

  while (1)
  {
    // if there's incoming data from the wind sensor.
    if (mySerial.available())
    {
      //read data from the wind sensor.
      char c = mySerial.read();
      //Show received sentence
      Serial.print(c);
      if ( Sequence == -1 )
      {
        if ( c == 'W' && prvc == '%' ){ Sequence = 0 ;}
      }
      else
      {
        //if the received data is <LF> then escape the loop
        if ( c == '\n' ){ break ;}
        else
        {
          //if the received data is ',' or '*' then proceed sequence
          if ( c == ',' || c == '*' )
          {
            Sequence++ ;
          }
          //if the Sequence is 1 then read instantaneous wind angle
          else if ( Sequence == 1 )
          {
            InstDir[0] = InstDir[1] ;
            InstDir[1] = InstDir[2] ;
            InstDir[2] = c ;
          }
          //if the Sequence is 3 then current wind speed letters shift in
          else if ( Sequence == 3 )
          {
            InstWin[0] = InstWin[1] ;
            InstWin[1] = InstWin[2] ;
            InstWin[2] = InstWin[3] ;
            InstWin[3] = InstWin[4] ;
            InstWin[4] = c ;
          }
          //if the Sequence is 5 then read status
          else if ( Sequence == 5 )
          {
            Status = c ;
          }
          //if the Sequence is 6 then Temperature letters shift in
          else if ( Sequence == 6 )
          {
            Temp[0] = Temp[1] ;
            Temp[1] = Temp[2] ;
            Temp[2] = Temp[3] ;
            Temp[3] = Temp[4] ;
            Temp[4] = Temp[5] ;
            Temp[5] = c ;
          }
          //if the Sequence is 8 then average wind direction letter shift in
          else if ( Sequence == 8 )
          {
            AvrDir[0] = AvrDir[1] ;
            AvrDir[1] = AvrDir[2] ;
            AvrDir[2] = c ;
          }
          //if the Sequence is 9 then average wind speed letter shift in
          else if ( Sequence == 9 )
          {
            AvrWin[0] = AvrWin[1] ;
            AvrWin[1] = AvrWin[2] ;
            AvrWin[2] = AvrWin[3] ;
            AvrWin[3] = AvrWin[4] ;
            AvrWin[4] = c ;
          }
          //if the Sequence is 10 then maximum wind direction letter shift in
          else if ( Sequence == 10 )
          {
            MaxDir[0] = MaxDir[1] ;
            MaxDir[1] = MaxDir[2] ;
            MaxDir[2] = c ;
          }
          //if the Sequence is 11 then maximum wind speed letter shift in
          else if ( Sequence == 11 )
          {
            MaxWin[0] = MaxWin[1] ;
            MaxWin[1] = MaxWin[2] ;
            MaxWin[2] = MaxWin[3] ;
            MaxWin[3] = MaxWin[4] ;
            MaxWin[4] = c ;
          }
          //if the Sequence is 12 then time stamp letters shift in
          else if ( Sequence == 12 )
          {
            TimeStamp[0] = TimeStamp[1] ;
            TimeStamp[1] = TimeStamp[2] ;
            TimeStamp[2] = TimeStamp[3] ;
            TimeStamp[3] = c ;
          }
        }
      }
      prvc = c ;
    }
  }
}

/*
 *Clear RX buffer
 *
*/
void ClearRXbuf()
{
	//clear the RX buffer
  while (1)
  {
    // if there's remaining data in buffer.
    if (mySerial.available())
    {
      //read data.
      char c = mySerial.read();
		}
		else
		{
			break;
		}
	}
}

/*
 *Convert wind direction angle value letters to cardinal letters. (when not mode 1 and not mode2)
 *Return 16 angle code in any mode.
*/
char ConvertDir( char *CDir )
{
  int i = atoi( CDir ) ;
  float num = 11.25 ;
  //calculate angle degree to angle number in float
  num = ( i + num ) / ( num * 2 ) ;
  //truncate to decimal point
  i = num ;
  //turn 16 to 0
  if ( i == 16 ) { i = 0 ;}
  char Ndir[] = " " ;
  sprintf ( Ndir, "%01X", i ) ;
  //if not mode 1 and not mode 2 then
  if ( ModeSel != 1 && ModeSel != 2)
  {
    //create average wind direction number from wind direction letters
    //0=N, 1=NNE, 2=NE, 3=ENE, 4=E, 5=ESE, 6=SE, 7=SSE, 8=S, 9=SSW, A=SW, B=WSW, C=W, D=WNW, E=NW, F=NNW
    //1 letter or 2 letter wind directions are right justified.
    //putting ' NE' first in the string for make following code minimum.
    CDir[0] = ' ' ; CDir[1] = 'N' ; CDir[2] = 'E' ;

    if ( i == 0 ) {                 CDir[1] = ' ' ; CDir[2] = 'N' ;}
    if ( i == 1 ) { CDir[0] = 'N' ;                                }

    if ( i == 3 ) { CDir[0] = 'E' ;                                }
    if ( i == 4 ) {                 CDir[1] = ' ' ;                }
    if ( i == 5 ) { CDir[0] = 'E' ; CDir[1] = 'S' ;                }
    if ( i == 6 ) {                 CDir[1] = 'S' ;                }
    if ( i == 7 ) { CDir[0] = 'S' ; CDir[1] = 'S' ;                }
    if ( i == 8 ) {                 CDir[1] = ' ' ; CDir[2] = 'S' ;}
    if ( i == 9 ) { CDir[0] = 'S' ; CDir[1] = 'S' ; CDir[2] = 'W' ;}
    if ( i == 10 ) {                 CDir[1] = 'S' ; CDir[2] = 'W' ;}
    if ( i == 11 ) { CDir[0] = 'W' ; CDir[1] = 'S' ; CDir[2] = 'W' ;}
    if ( i == 12 ) {                 CDir[1] = ' ' ; CDir[2] = 'W' ;}
    if ( i == 13 ) { CDir[0] = 'W' ;                 CDir[2] = 'W' ;}
    if ( i == 14 ) {                                 CDir[2] = 'W' ;}
    if ( i == 15 ) { CDir[0] = 'N' ;                 CDir[2] = 'W' ;}
  }
  return Ndir[0] ;
}

// this method makes a HTTP connection to the server:
void httpRequest()
{
  // close any connection before send a new request.
  // This will free the socket on the WiFi shield
  client.stop();
  // if there's a successful connection:
  if (client.connect(server, 80))
  {
#ifdef DEBUG_HTTP
    Serial.println(server);
    Serial.println(F("connecting..."));
#endif
    // send the HTTP PUT request:
    client.print(F("GET /"));
    client.print(dir);
    client.print(F("wlog.cgi?DTMF="));
    for ( char i=0; i<9; i++ )
    {
      client.print(DTMFhex[i]);
    }
    client.println(F(" HTTP/1.1"));
    client.print(F("Host: "));
    client.println(server);
    client.println(F("User-Agent: arduino-ethernet"));
    client.println(F("Connection: close"));
    client.println();

    ConnectionSuccess = true ;
  } 
  else
  {
    // if you couldn't make a connection:
#ifdef DEBUG_HTTP
    Serial.println(F("connection failed"));
#endif
    ConnectionSuccess = false;
  }
  // note the time that the attempt of connection was made:
  lastAttemptTime = millis();
}

char bin2ascii( char chr )
{
  chr &= 0x0f ;
  if ( chr < 10 )
  {
    chr |= 0x30 ;
  }
  else
  {
    chr += 0x37 ;
  }
  return chr ;
}

